@@ -101,6 +101,7 @@
 //!   - Dependencies are full of unsafe
 //!
 //! # Changelog
+//! - v0.3.2 - Re-export `safina_sync::Receiver` and `safina_threadpool::NewThreadPoolError`.
 //! - v0.3.1 - Use `safina-async` v0.2.1.
 //! - v0.3.0 - `schedule_blocking` to return new `safina_sync::Receiver`.
 //! - v0.2.1 - Update docs.
@@ -143,8 +144,9 @@ use core::cell::Cell;
 use core::future::Future;
 use core::pin::Pin;
 use core::task::Poll;
-use safina_sync::Receiver;
-use safina_threadpool::{NewThreadPoolError, ThreadPool};
+pub use safina_sync::Receiver;
+pub use safina_threadpool::NewThreadPoolError;
+use safina_threadpool::ThreadPool;
 use std::sync::mpsc::SyncSender;
 use std::sync::{Arc, Mutex, Weak};
 
@@ -405,7 +407,10 @@ where
     if let Some(executor) = get_thread_executor() {
         executor.schedule_blocking(func)
     } else {
-        panic!("called from outside a task")
+        // See explanation in `spawn_unpin`.
+        panic!(
+            "called from outside a task; check for duplicate safina-executor crate: cargo tree -d"
+        );
     }
 }
 
@@ -496,7 +501,14 @@ pub fn spawn_unpin(fut: impl (Future<Output = ()>) + Send + Unpin + 'static) {
             .async_pool
             .schedule(move || poll_task(task, executor_weak));
     } else {
-        panic!("called from outside a task");
+        // Note: This panic can happen when there are two versions of the safina-executor crate.
+        // Example:  You use the `async_test` macro.  It creates an executor with the first
+        // version of the crate.  The executor is stored in the first crate's thread-local storage.
+        // The second crate's thread-local storage is empty.  If the test code calls the second
+        // crate's spawn or schedule functions, it hits this error.
+        panic!(
+            "called from outside a task; check for duplicate safina-executor crate: cargo tree -d"
+        );
     }
 }
 
