@@ -96,15 +96,32 @@ async fn expect_blocking_concurrency(executor: &Arc<Executor>, num_tasks: usize)
     );
 }
 
-fn expect_elapsed(before: Instant, range: Range<Duration>) {
-    assert!(!range.is_empty(), "invalid range {:?}", range);
+trait UnwindAnyToString {
+    fn any_to_string(&self) -> String;
+}
+impl UnwindAnyToString for Box<dyn std::any::Any + Send> {
+    fn any_to_string(&self) -> String {
+        if let Some(s) = self.downcast_ref::<&'static str>() {
+            (*s).to_string()
+        } else if let Some(s) = self.downcast_ref::<String>() {
+            s.clone()
+        } else {
+            format!("{:?}", self)
+        }
+    }
+}
+
+/// # Panics
+/// Panics if the time elapsed since `before` is not in `range_ms`.
+pub fn expect_elapsed(before: Instant, range_ms: Range<u64>) {
+    assert!(!range_ms.is_empty(), "invalid range {:?}", range_ms);
     let elapsed = before.elapsed();
-    println!("{:?} elapsed, expected range {:?}", elapsed, range);
+    let duration_range = Duration::from_millis(range_ms.start)..Duration::from_millis(range_ms.end);
     assert!(
-        range.contains(&elapsed),
+        duration_range.contains(&elapsed),
         "{:?} elapsed, out of range {:?}",
         elapsed,
-        range
+        duration_range
     );
 }
 
@@ -250,10 +267,7 @@ fn test_waking() {
         sender.send(()).unwrap();
     });
     receiver.recv().unwrap();
-    expect_elapsed(
-        before,
-        Duration::from_millis(100)..Duration::from_millis(200),
-    );
+    expect_elapsed(before, 100..200);
 }
 
 #[test]
@@ -261,9 +275,7 @@ fn test_async_panic() {
     // Start a single thread.
     let executor = safina_executor::Executor::new(1, 1).unwrap();
     // Run a future that panics.
-    executor.spawn(async move {
-        panic!("ignore this panic");
-    });
+    executor.spawn(async { panic!("ignore this panic") });
     std::thread::sleep(Duration::from_millis(100));
     // Check that futures still run.
     let (sender, receiver) = std::sync::mpsc::channel();
@@ -284,20 +296,14 @@ fn test_executor_block_on() {
     executor.block_on(async {
         safina_timer::sleep_for(Duration::from_millis(100)).await;
     });
-    expect_elapsed(
-        before,
-        Duration::from_millis(100)..Duration::from_millis(200),
+    expect_elapsed(before, 100..200);
+    assert_eq!(
+        "ignore this panic",
+        std::panic::catch_unwind(|| executor.block_on(async { panic!("ignore this panic") }))
+            .unwrap_err()
+            .any_to_string()
+            .as_str()
     );
-    std::panic::catch_unwind(|| {
-        executor.block_on(async {
-            #[allow(clippy::manual_assert)]
-            if true {
-                panic!("ignore this panic");
-            }
-            1_u8
-        });
-    })
-    .unwrap_err();
     executor.block_on(async {
         let (sender, receiver) = std::sync::mpsc::channel();
         safina_executor::spawn(async move {
@@ -317,28 +323,16 @@ fn test_block_on() {
     safina_executor::block_on(async {
         safina_timer::sleep_for(Duration::from_millis(100)).await;
     });
-    expect_elapsed(
-        before,
-        Duration::from_millis(100)..Duration::from_millis(200),
+    expect_elapsed(before, 100..200);
+    assert_eq!(
+        "ignore this panic",
+        std::panic::catch_unwind(|| safina_executor::block_on(async {
+            panic!("ignore this panic");
+        }))
+        .unwrap_err()
+        .any_to_string()
+        .as_str()
     );
-    std::panic::catch_unwind(|| {
-        safina_executor::block_on(async {
-            #[allow(clippy::manual_assert)]
-            if true {
-                panic!("ignore this panic");
-            }
-            1_u8
-        });
-    })
-    .unwrap_err();
-}
-
-#[test]
-fn test_block_on_spawn_error() {
-    std::panic::catch_unwind(|| {
-        safina_executor::block_on(async { safina_executor::spawn(async {}) });
-    })
-    .unwrap_err();
 }
 
 #[test]
@@ -424,10 +418,7 @@ fn test_schedule_blocking_sleep() {
             .await
             .unwrap()
         );
-        expect_elapsed(
-            before,
-            Duration::from_millis(100)..Duration::from_millis(200),
-        );
+        expect_elapsed(before, 100..200);
     });
 }
 #[test]
@@ -447,10 +438,7 @@ fn test_executor_schedule_blocking_sleep() {
                 .await
                 .unwrap()
         );
-        expect_elapsed(
-            before,
-            Duration::from_millis(100)..Duration::from_millis(200),
-        );
+        expect_elapsed(before, 100..200);
     });
 }
 
@@ -458,16 +446,10 @@ fn test_executor_schedule_blocking_sleep() {
 fn test_schedule_blocking_panic() {
     let executor = safina_executor::Executor::new(1, 1).unwrap();
     executor.block_on(async {
-        safina_executor::schedule_blocking(|| {
-            #[allow(clippy::manual_assert)]
-            if true {
-                panic!("ignore this panic");
-            }
-            1_u8
-        })
-        .async_recv()
-        .await
-        .unwrap_err();
+        safina_executor::schedule_blocking(|| panic!("ignore this panic"))
+            .async_recv()
+            .await
+            .unwrap_err();
     });
 }
 #[test]
@@ -476,13 +458,7 @@ fn test_executor_schedule_blocking_panic() {
     let executor_clone = executor.clone();
     executor.block_on(async move {
         executor_clone
-            .schedule_blocking(|| {
-                #[allow(clippy::manual_assert)]
-                if true {
-                    panic!("ignore this panic");
-                }
-                1_u8
-            })
+            .schedule_blocking(|| panic!("ignore this panic"))
             .async_recv()
             .await
             .unwrap_err();
@@ -501,10 +477,7 @@ fn test_schedule_blocking_recv() {
         safina_timer::sleep_for(Duration::from_millis(100)).await;
         sender.send(9_u8).unwrap();
         assert_eq!(9_u8, result_receiver.async_recv().await.unwrap());
-        expect_elapsed(
-            before,
-            Duration::from_millis(100)..Duration::from_millis(200),
-        );
+        expect_elapsed(before, 100..200);
     });
 }
 #[test]
@@ -520,15 +493,28 @@ fn test_executor_schedule_blocking_recv() {
         safina_timer::sleep_for(Duration::from_millis(100)).await;
         sender.send(9_u8).unwrap();
         assert_eq!(9_u8, result_receiver.async_recv().await.unwrap());
-        expect_elapsed(
-            before,
-            Duration::from_millis(100)..Duration::from_millis(200),
-        );
+        expect_elapsed(before, 100..200);
     });
 }
 
 #[test]
-#[should_panic]
 fn no_executor_running() {
-    safina_executor::schedule_blocking(|| 3_u8);
+    assert_eq!(
+        "called from outside a task; check for duplicate safina-executor crate: cargo tree -d",
+        std::panic::catch_unwind(
+            || safina_executor::block_on(safina_executor::schedule_blocking(|| 3_u8))
+        )
+        .unwrap_err()
+        .any_to_string()
+        .as_str()
+    );
+    assert_eq!(
+        "called from outside a task; check for duplicate safina-executor crate: cargo tree -d",
+        std::panic::catch_unwind(|| safina_executor::block_on(async {
+            safina_executor::spawn(async {});
+        }))
+        .unwrap_err()
+        .any_to_string()
+        .as_str()
+    );
 }
